#include "CLED.h"
#include <SPI.h>
// ************************* Object ********************************
extern CLED cled;
// *************************** GPIO ********************************
const int abc[3] = {2,3,4};
/*Arduino Nano Every			   IHS11 双方向ハンドセンサー(Interactive Hand Sensor)
Select
  const int A = 2;            	   3:A ch: 0bit
  const int B = 3;            	   4:B ch: 1bit
  const int C = 4;            	   5:C ch: 2bit
SPI
  SS:   pin 8              	       6:SS
  MOSI: pin 13　                   7:MOSI
  MISO: pin 12　                   8:MISO
  SCK:  pin 11                     9:SCK  */
extern const int SEL_SEN = 5; 	// 10:Select Sensor,		HIGH:Sensor, LOW:CLED or other
extern const int OE_ILED = 6; 	// 11:Output Enable ILED	HIGH:ILED ON, LOW:ILED OFF (Infrared LED)
extern const int FIX_CLED = 7; 	// 12:FIX CLED				LOW->HIGH:Fix CLED
extern int OE_CLED = 10;    	// 13,Output Enable CLED	CLED output enable: OE_CLED and PWM_CLED (13 and 14)
extern const int PWM_CLED = 9;	// 14:PWM CLED				CLED output enable: OE_CLED and PWM_CLED (13 and 14)
// ************************* local function ***************************** 
int noMinus(int n) {
	if (n < 0) return 0;
	return n;
}

void out3bit(int ary[], int val) {
	for (int i = 0; i < 3; i++)
		digitalWrite(ary[i], (val >> i) & 0x01);
}
// **********************************************************************
int Sensor::cri[CRI_LEN] = {135,290,550,1100,1900,3200}; // criteria
long int Sensor::adAryInit[COL_LEN];

Sensor::Sensor(int dmy) {}					// dmy: Error countermeasures

void Sensor::init() {
	const int CORRECTION = 20;
    for (int i = 0; i < CRI_LEN; i++) {
        cri[i] += CORRECTION;
    }
	for (int i=0; i<sizeof(abc); i++) {
		pinMode(abc[i], OUTPUT);	
	}
	pinMode(SEL_SEN, OUTPUT);
	pinMode(SS, OUTPUT);
	pinMode(OE_ILED, OUTPUT);
	pinMode(FIX_CLED, OUTPUT);
	pinMode(OE_CLED, OUTPUT); 
	pinMode(PWM_CLED, OUTPUT);
	analogWrite(PWM_CLED, 150);   // 0 ~ 255(max)
    for (int i = 0; i < COL_LEN; i++) {
        adAry[i] = 0;
    }
	SPI.begin();
	SPI.beginTransaction(SPISettings(2e6, MSBFIRST, SPI_MODE0));//1600000
	set_adAryInit();
}

void Sensor::setAd(bool bInit) {            // bInit:add val to adAryInit[]
	for (int col = 0; col < COL_LEN; col++) {
		setCol(col, bInit);					// *** sensing ***
	}
	if (!bInit)                             // *** set CLED ***
		cled.set(val);						// sensor-data, indicator
}

int Sensor::getColVal(int c) {
	return val[c];
}

void Sensor::getColVal(int* ary) {
	for (int i = 0; i < COL_LEN; i++)
		ary[i] = val[i];          			// copy val to ary
}

int Sensor::getColAd(int c) {
	return adAry[c];
}

void Sensor::getColAd(int* ary) {
	for (int i = 0; i < COL_LEN; i++)
		ary[i] = adAry[i];          		// copy adAry to ary
}

void Sensor::getMaxAd(int iAd[2]) {			// iAd[idx, AD value]
	iAd[0] = iAd[1] = 0;
	for (int i = 0; i < COL_LEN; i++) {
		if (adAry[i] > iAd[1]) {
			iAd[0] = i;
			iAd[1] = adAry[i];
		}
	}
}

void Sensor::set_adAryInit() {
	const int INIT_MS = 1200;
	int cnt = 0;
// ***************** set adAryInit[] value ******************
	unsigned long stt = millis();
	while (millis() - stt < INIT_MS) {
			setAd(true);
			cnt++;
	}
	for (int col = 0; col < COL_LEN; col++) {
		adAryInit[col] = adAryInit[col] / cnt;
	}
}

unsigned int Sensor::getAdc(int col) {
	digitalWrite(SS, LOW);
	SPI.transfer(6 | (col >> 2));//
	unsigned char r1 = SPI.transfer(col << 6);      
	unsigned char r2 = SPI.transfer(0);
	digitalWrite(SS, HIGH);
	return ((r1 & 0x0f) << 8) + r2;
}

void Sensor::setCol(int col, bool bInit) {
	int ledSta = digitalRead(OE_CLED);
	digitalWrite(OE_CLED, LOW);
	out3bit(abc, col);
	digitalWrite(SEL_SEN, HIGH);
	digitalWrite(OE_ILED, LOW);     // ILED-OFF
	int vOff = getAdc(col);			// *** read ADC ***
	digitalWrite(OE_ILED, HIGH);    // ILED-ON
	int vOn = getAdc(col);      	// *** read ADC ***
	digitalWrite(SEL_SEN, LOW);
	digitalWrite(OE_ILED, LOW);     // ILED-OFF
	digitalWrite(OE_CLED, ledSta);
	if (bInit)
		adAryInit[col] += vOn - vOff - (vOff >> 2) + (vOn >> 4);
	else
		adAry[col] = noMinus(vOn - vOff - (vOff >> 2) + (vOn >> 4) - adAryInit[col]);
	val[col] = ad2val(adAry[col]);
}
int Sensor::ad2val(int ad) {  // div6 true:1~6 false:1~12
	for (int i = 0; i < CRI_LEN; i++)
		if (ad < cri[i]) return i;
	return CRI_LEN;
}
